<?php
/**
 * @author Clark Tomlinson  <clark@owncloud.com>
 * @since 10/30/14, 1:22 PM
 * @copyright 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/
 */

namespace OCA\Firewall\AppInfo;

require_once __DIR__ . '/../3rdparty/autoload.php';


use Hoa\Ruler\Ruler;
use OCA\Firewall\Config;
use OCA\Firewall\Events\Listeners\FailureListener;
use OCA\Firewall\Events\Subscribers\LoggerSubscriber;
use OCA\Firewall\Firewall;
use OCA\Firewall\Rules;
use OCP\AppFramework\App;
use OCP\App as PApp;
use OCP\AppFramework\IAppContainer;

class Application extends App {

	/**
	 * @param array $urlParams
	 */
	public function __construct(array $urlParams = array()) {
		parent::__construct('firewall', $urlParams);

		$this->registerServices();
	}

	/**
	 * Register any services into the DI Container
	 */
	protected function registerServices() {
		$this->getContainer()->registerService('FirewallConfig', function () {
			return new Config(\OC::$SERVERROOT . '/config/');
		});

		$this->getContainer()->registerService('Ruler', function () {
			return new Ruler();
		});

		$this->getContainer()->registerService('MobileDetect', function () {
			return new \Mobile_Detect();
		});
	}

	/**
	 * Boot the firewall application
	 */
	public function boot() {
		$this->registerSettings();
		$this->startFirewall($this->getContainer());
	}

	/**
	 * Register Any needed settings pages
	 */
	public function registerSettings() {
		// register settings page
		PApp::registerAdmin('firewall', 'settings/settings');
	}

	/**
	 * Boot the firewall along with any needed subscribers and listeners
	 * Todo: Separate listeners and subscribers and make them extensible to public
	 *
	 * @param IAppContainer $container
	 */
	private function startFirewall(IAppContainer $container) {

		$server = $container->getServer();

		$subscribers = array(
			new LoggerSubscriber($server->getLogger(), $container->query('FirewallConfig')->getDebugLevel()),
		);

		$listeners = array(
			'firewall.fail' => array(new FailureListener(), 'onFail')
		);

		$rulesObj = $container->query('FirewallConfig')->getRules(true);
		if (!empty($rulesObj)) {
			$rules = new Rules($rulesObj, $container->query('Ruler'), $container->query('MobileDetect'), $server->getRequest());
			$firewall = new Firewall($rules, $listeners, $subscribers, $server->getUserSession(), $server->getGroupManager(), $server->getRequest());
			// Check Request
			$firewall->checkRequest();
		}
	}

}
