<?php
/**
 * @author Clark Tomlinson  <clark@owncloud.com>
 * @since 10/30/14, 1:25 PM
 * @copyright 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/
 */

namespace OCA\Firewall\Controller;


use OC\Group\MetaData;
use OCA\Firewall\Config;
use OCA\Firewall\Rules;
use OCP\AppFramework\Controller;
use OCP\AppFramework\Http;
use OCP\AppFramework\Http\DataResponse;
use OCP\IGroupManager;
use OCP\IRequest;
use OCP\IUserSession;


class RulesController extends Controller {

	/**
	 * @var IUserSession
	 */
	private $userSession;
	/**
	 * @var IGroupManager
	 */
	private $groupManager;
	/**
	 * @var Config
	 */
	private $config;

	/**
	 * @param string $AppName
	 * @param IRequest $request
	 * @param Config $FirewallConfig
	 * @param IUserSession $userSession
	 * @param IGroupManager $groupManager
	 */
	public function __construct($AppName, IRequest $request, $FirewallConfig, IUserSession $userSession, IGroupManager $groupManager) {
		parent::__construct($AppName, $request);
		$this->userSession = $userSession;
		$this->groupManager = $groupManager;
		$this->config = $FirewallConfig;
	}

	/**
	 * Saves rules to persistent storage
	 *
	 * @param $rules
	 * @return DataResponse
	 */
	public function save($rules) {

		try {
			$this->config->setRules($rules);
			return new DataResponse(array(), Http::STATUS_OK);
		} catch (\Exception $e) {
			return new DataResponse(array('message' => $e->getMessage()), Http::STATUS_INTERNAL_SERVER_ERROR);
		}
	}

	/**
	 * Set our debugLevel
	 *
	 * @param $level
	 * @return DataResponse
	 */
	public function debug($level) {
		try {
			$this->config->setDebug($level);
			return new DataResponse(array(), Http::STATUS_OK);
		} catch (\Exception $e) {
			return new DataResponse(array('message' => $e->getMessage()), Http::STATUS_INTERNAL_SERVER_ERROR);
		}
	}

	/**
	 * Return rules and debug level for the ui to consume
	 *
	 * @return DataResponse
	 */
	public function getUIData() {
		return new DataResponse(array(
			'rules' => $this->config->getRules(),
			'debugLevel' => $this->config->getDebugLevel()
		), Http::STATUS_OK);
	}

	/**
	 * @param string $pattern
	 * @param bool $filterGroups
	 * @return DataResponse
	 */
	public function getGroups($pattern = '', $filterGroups = false) {
		$groupPattern = $filterGroups ? $pattern : '';

		$groupsInfo = new MetaData($this->userSession->getUser()->getUID(),
			$this->groupManager->isAdmin($this->userSession->getUser()->getUID()), $this->groupManager);
		$groupsInfo->setSorting($groupsInfo::SORT_USERCOUNT);
		list($adminGroups, $groups) = $groupsInfo->get($groupPattern, $pattern);

		return new DataResponse(
			array(
				'data' => array('adminGroups' => $adminGroups, 'groups' => $groups)
			)
		);
	}
}
