<?php
/**
 * @author Clark Tomlinson  <clark@owncloud.com>
 * @since 11/19/14, 11:20 AM
 * @copyright 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/
 */

namespace OCA\Firewall\Events\Contracts;


use Symfony\Component\EventDispatcher\EventDispatcher as Dispatcher;

trait EventDispatcher {

	/**
	 * @var Dispatcher
	 */
	protected $dispatcher;

	public function __construct() {
		$this->boot();
	}

	public function boot() {
		$this->dispatcher = new Dispatcher();
	}

	public function addObservers($subscriber, $listeners) {
		$this->boot();
		$this->attachSubscribers($subscriber);
		$this->attachListeners($listeners);
		return $this;
	}

	private function attachSubscribers($subscribers) {
		foreach ($subscribers as $subscriber) {
			if (!$subscriber instanceof Subscriber) {
				throw new \Exception('Passed Observer does not implement Subscriber Interface');
			}

			$this->dispatcher->addSubscriber($subscriber);
		}
	}

	private function attachListeners($listeners) {
		foreach ($listeners as $event => $listener) {
			if (!is_callable($listener) && $listener instanceof EventListener) {
				throw new \Exception('Passed Observer does not implement EventListener Interface');
			}

			$this->dispatcher->addListener($event, $listener);
		}
	}

}