<?php
/**
 * @author Clark Tomlinson  <clark@owncloud.com>
 * @since 11/25/14, 11:37 AM
 * @copyright 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/
 *
 */

namespace OCA\Firewall\Events\Contracts;


use Symfony\Component\EventDispatcher\Event;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

abstract class Subscriber implements EventSubscriberInterface {
	/**
	 * Returns an array of event names this subscriber wants to listen to.
	 *
	 * The array keys are event names and the value can be:
	 *
	 *  * The method name to call (priority defaults to 0)
	 *  * An array composed of the method name to call and the priority
	 *  * An array of arrays composed of the method names to call and respective
	 *    priorities, or 0 if unset
	 *
	 * For instance:
	 *
	 *  * array('eventName' => 'methodName')
	 *  * array('eventName' => array('methodName', $priority))
	 *  * array('eventName' => array(array('methodName1', $priority), array('methodName2'))
	 *
	 * @return array The event names to listen to
	 *
	 * @api
	 */
	public static function getSubscribedEvents() {

		return array(
			'firewall.pass' => array('onPass', 0),
			'firewall.fail' => array('onFail', 0),
		);
	}

	/**
	 * Add your logic for when rules pass
	 * @param $event
	 */
	public abstract function onPass(Event $event);

	/**
	 * Add your logic for when their is a failure
	 * @param $event
	 */
	public abstract function onFail(Event $event);
}