<?php
/**
 * @author Clark Tomlinson  <clark@owncloud.com>
 * @since 11/25/14, 11:33 AM
 * @copyright 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/
 */

namespace OCA\Firewall\Events\Subscribers;

use OCA\Firewall\Events\Contracts\Subscriber;
use OCP\ILogger;
use Symfony\Component\EventDispatcher\Event;

class LoggerSubscriber extends Subscriber {
	/**
	 * @var int
	 */
	protected $logLevel;

	/**
	 * @var ILogger
	 */
	private $log;

	/**
	 * @param ILogger $log
	 * @param int $logLevel
	 */
	public function __construct(ILogger $log, $logLevel) {
		$this->log = $log;
		$this->logLevel = $logLevel;
	}

	public function onPass(Event $event) {

		if ($this->logLevel === 2) {
			$outcome = $event->getRuleOutcomes();
			$this->log->debug("Rule outcomes: {$outcome}", array('app' => 'firewall'));
		}
	}

	public function onFail(Event $event) {
		$offender = explode(':', $event->getOffender());
		$this->log->debug("Rule failure on group {$offender['0']} rule {$offender['1']}", array('app' => 'firewall'));
	}


}