$(function () {

	// Cache Rulebuilder selector
	var $ruleBuilder = $('#rulebuilder');


	// Set localization for timepicker
	$.timepicker.setDefaults($.timepicker.regional[$('html').prop('lang')]);

	// Extend and add new methods
	$.extend($.fn.queryBuilder.constructor.prototype, {

		/**
		 * Perform action when rule's filter is changed
		 * @param $rule {jQuery} (<li> element)
		 * @param filterId {string}
		 */
		updateRuleFilter: function ($rule, filterId) {
			var filter = filterId != '-1' ? this.getFilterById(filterId) : null;

			var $regexValue = $rule.find('input[name=regexValue]');
			if (filterId !== 'regex' && $regexValue) {
				$regexValue.remove();
			}

			this.createRuleOperators($rule, filter);
			this.createRuleInput($rule, filter);

			$rule.data('queryBuilder.filter', filter);
		},
		/**
		 *
		 * @returns {*|QueryBuilder.DEFAULTS.filters|Array|QueryBuilder.filters|Sizzle.selectors.filters|fb.selectors.filters}
		 */
		getFilters: function () {
			return this.filters;
		},
		/**
		 * Get an object representing current rules
		 * @return {object}
		 */
		getRules: function () {
			this.markRuleAsError(this.$el.find('.rule-container'), false);

			var $groups = this.$el.find('>.rules-group-container'),
				mainout = [],
				that = this;

			$groups.each(function (k, group) {
				mainout.push((function parse (group) {
					var out = {},
						$group = $(group),
						$elements = $group.find('>.rules-group-body>.rules-list>*');

					out.name = $group.find('.groupName').val();
					out.condition = $group.find('>.rules-group-header input[name$=_cond]:checked').val();
					out.rules = [];

					for (var i = 0, l = $elements.length; i < l; i++) {
						var $rule = $elements.eq(i),
							rule;

						if ($rule.hasClass('rule-container')) {

							var filterId = that.getRuleFilter($rule);

							if (filterId == '-1') {
								continue;
							}

							var filter = that.getFilterById(filterId),
								operator = that.getOperatorByType(that.getRuleOperator($rule)),
								value = null;

							if (operator.accept_values !== 0) {
								value = that.getRuleValue($rule, filter, operator);


								if (filter.valueParser) {
									value = filter.valueParser.call(this, $rule, value, filter, operator);
								}

								var valid = that.validateValue($rule, value, filter, operator);
								if (valid !== true) {
									that.markRuleAsError($rule, true);
									that.triggerValidationError(valid, $rule, value, filter, operator);
									return {};
								}
							}

							rule = {
								id: filter.id,
								field: filter.field,
								type: filter.type,
								input: filter.input,
								operator: operator.type,
								value: value
							};

							out.rules.push(rule);
						}
						else {
							rule = parse($rule);
							if (!$.isEmptyObject(rule)) {
								out.rules.push(rule);
							}
							else {
								return {};
							}
						}
					}

					if (out.rules.length === 0) {
						that.triggerValidationError('empty_group', $group, null, null, null);

						return {};
					}

					return out;
				}(group)));
			});
			return mainout;

		},
		/**
		 * Set rules from object
		 * @param data {object}
		 */
		setRules: function (data) {
			this.clear();

			var $container = this.$el,
				that = this;

			$(data).each(function (k, data) {
				if (!data || !data.rules || data.rules.length === 0) {
					$.error('Incorrect data object passed');
				}

				(function add (data, $container) {
					var $group = that.addGroup($container, false),
						$ul = $group.find('>.rules-group-body>.rules-list'),
						$buttons = $group.find('>.rules-group-header input[name$=_cond]');

					if (data.condition === undefined) {
						data.condition = that.settings.default_condition;
					}

					for (var i = 0, l = that.settings.conditions.length; i < l; i++) {
						var cond = that.settings.conditions[i];
						$buttons.filter('[value=' + cond + ']').prop('checked', data.condition.toUpperCase() == cond.toUpperCase());
					}
					$buttons.trigger('change');

					$group.find('.groupName').val(data.name);

					$.each(data.rules, function (i, rule) {
						if (rule.rules && rule.rules.length > 0) {
							if (!that.settings.allow_groups) {
								$.error('Groups are disabled');
							}
							else {
								add(rule, $container);
							}
						}
						else {
							if (rule.id === undefined) {
								$.error('Missing rule field id');
							}
							if (rule.value === undefined) {
								rule.value = '';
							}
							if (rule.operator === undefined) {
								rule.operator = 'equal';
							}

							var $rule = that.addRule($ul),
								filter = that.getFilterById(rule.id),
								operator = that.getOperatorByType(rule.operator);

							$rule.find('.rule-filter-container select[name$=_filter]').val(rule.id).trigger('change');
							$rule.find('.rule-operator-container select[name$=_operator]').val(rule.operator).trigger('change');

							if (operator.accept_values !== 0) {
								that.setRuleValue($rule, rule, filter, operator);
							}

							if (filter.onAfterSetValue) {
								filter.onAfterSetValue.call(that, $rule, rule.value, filter, operator);
							}
						}
					});

				}(data, $container));
			});
		},
		/**
		 * Returns group HTML
		 * @param group_id {string}
		 * @param main {boolean}
		 * @return {string}
		 */
		getGroupTemplate: function (group_id, main) {
			var h = '\
				<dl id="' + group_id + '" class="rules-group-container" ' + (this.settings.sortable ? 'draggable="true"' : '') + '> \
				  <dt class="rules-group-header"> \
				  	' + (this.settings.sortable ? '<div class="drag-handle"><i class="' + this.settings.icons.sort + '"></i></div>' : '') + ' \
					<div class="btn-group"> \
					  ' + this.getGroupConditions(group_id) + ' \
						<input type="text" name="groupName" class="groupName" placeholder="Group Name"/> \
					</div> \
				  </dt> \
				  <dd class=rules-group-body> \
					<ul class=rules-list></ul> \
				  </dd> \
				  <dt>\
					<div class="btn-group management-btns pull-right"> \
					<button type="button" class="btn btn-xs btn-success" data-add="rule"> \
					<i class="' + this.settings.icons.add_rule + '"></i> ' + this.lang.add_rule + ' \
					</button> \
							  ' + (this.settings.allow_groups ? '<button type="button" class="btn btn-xs btn-success" data-add="group"> \
					<i class="' + this.settings.icons.add_group + '"></i> ' + this.lang.add_group + ' \
					</button>' : '') + ' \
							  ' + (!main ? '<button type="button" class="btn btn-xs btn-danger" data-delete="group"> \
					<i class="' + this.settings.icons.remove_group + '"></i> ' + this.lang.delete_group + ' \
					</button>' : '') + ' \
					</div> \
				</dt>\
				</dl>';

			return h;
		}
	});

	// Local storage if no other medium is available
	var storage = (typeof(sessionStorage) === 'undefined') ?
		(typeof(localStorage) === 'undefined') ? {
			store: {},
			getItem: function (key) {
				return this.store[key];
			},
			setItem: function (key, value) {
				this.store[key] = value;
			},
			removeItem: function (key) {
				delete this.store[key];
			},
			clear: function () {
				for (var key in this.store) {
					if (this.store.hasOwnProperty(key)) {
						delete this.store[key];
					}
				}
			}
		} : localStorage : sessionStorage;

	// Allows us to cache ajax requests
	$.ajaxTransport("json", function (options) {
		if (options.localCache) {
			var cacheKey = options.cacheKey ||
				options.url.replace(/jQuery.*/, '') + options.type + options.data;

			var value = storage.getItem(cacheKey);
			if (value) {
				// In the cache? Get it, parse it to json, call the completeCallback with the fetched value.
				if (options.dataType.indexOf('json') === 0) {
					value = JSON.parse(value);
				}
				return {
					send: function (headers, completeCallback) {
						completeCallback(200, 'success', {json: value});
					}
				};
			}
		}
	});

	$.ajaxPrefilter(function (options, originalOptions, jqXHR) {
		if (!storage || !options.localCache) {
			return;
		}

		// Default cache to 5 minutes if its not defined
		var minutestl = options.cacheTTL || 5;

		var cacheKey = options.cacheKey ||
			options.url.replace(/jQuery.*/, '') + options.type + options.data;

		// isCacheValid is a function to validate cache
		if (options.isCacheValid && !options.isCacheValid()) {
			storage.removeItem(cacheKey);
		}
		// if there's a TTL that's expired, flush this item
		var ttl = storage.getItem(cacheKey + 'cachettl');
		if (ttl && ttl < +new Date()) {
			storage.removeItem(cacheKey);
			storage.removeItem(cacheKey + 'cachettl');
			ttl = 'expired';
		}

		var value = storage.getItem(cacheKey);
		if (value) {
			// In the cache? so get it, apply success callback & abort the XHR request
			// Parse back to JSON if we can
			if (options.dataType.indexOf('json') === 0) {
				value = JSON.parse(value);
				$.when($(this).trigger('ajaxSuccess')).done(function () {
					return jqXHR.done(value);
				});
			}

		} else {
			// If it not in the cache, we store the data, add success callback - normal callback will proceed
			if (options.success) {
				options.realsuccess = options.success;
			}
			options.success = function (data) {
				var strdata = data;
				if (this.dataType.indexOf('json') === 0) {
					strdata = JSON.stringify(data);
				}

				// Save the data to storage catching exceptions (possibly QUOTA_EXCEEDED_ERR)
				try {
					storage.setItem(cacheKey, strdata);
				} catch (e) {
					// Remove any incomplete data that may have been saved before the exception was caught
					storage.removeItem(cacheKey);
					storage.removeItem(cacheKey + 'cachettl');
					if (options.cacheError) {
						options.cacheError(e, cacheKey, strdata);
					}
				}

				if (options.realsuccess) {
					return options.realsuccess(data);
				}
			};

			// store timestamp
			if (!ttl || ttl === 'expired') {
				storage.setItem(cacheKey + 'cachettl', +new Date() + 1000 * minutestl);
			}
		}
	});

	var ruleCount = 1;


	$ruleBuilder.queryBuilder({
		sortable: true,
		lang: {
			"operator_equal": "is",
			"operator_not_equal": "is not"
		},
		icons: {
			add_group: 'fa fa-plus',
			add_rule: 'fa fa-plus',
			remove_group: 'fa fa-remove',
			remove_rule: 'fa fa-remove',
			sort: 'fa fa-exchange fa-rotate-90',
			error: 'fa fa-warning-sign'
		},
		onAfterAddGroup: function ($group) {
			// We need to reset the rule numbers for each group
			ruleCount = 1;
		},

		onAfterAddRule: function ($rule) {
			$rule.find('.drag-handle').append(ruleCount);
			ruleCount++;
		},


		filters: [
			/*
			 * basic
			 */
			{
				id: 'regex',
				label: 'Regular Expression',
				type: 'string',
				input: function ($rule, filter) {
					// Move text area before operator
					if (!$rule.find('[name=regexValue]').length) {
						$rule.find('.rule-filter-container').append('<input type="text" name="regexValue">');
						$rule.find('.rule-operator-container').append('<select class="regexRules"></select>');
					}
					buildRegexSelect();
				},
				onAfterSetValue: function ($rule, value, filter, operator) {
					var parts = value.split('||');
					$rule.find('[name=regexValue]').val(parts[1]);
					$rule.find('.regexRules').val(parts[0]);
				},
				operators: ['equal', 'not_equal'],
				valueParser: function ($rule, value, filter, operator) {
					return $rule.find('.regexRules').val() + '||' + $rule.find('[name=regexValue]').val();

				}
			},

			/*
			 * CIDR
			 */
			{
				id: 'cidr',
				label: 'Request IP Range',
				type: 'string',
				validation: {
					format: /^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])(\/(\d|[1-2]\d|3[0-2]))$/
				},
				placeholder: '127.0.0.1/24',
				operators: ['equal', 'not_equal']
			},

			/*
			 * Upload Size
			 */

			{
				id: 'sizeup',
				label: 'Max Upload Size',
				type: 'string',
				input: 'text',
				placeholder: '10m',
				operators: ['equal', 'not_equal', 'less', 'less_or_equal', 'greater', 'greater_or_equal'],
				validation: {
					format: /(\d+(?:\.\d+)?)\s?(k|m|g|t)?b?/i
				},
				valueParser: function ($rule, value, filter, operator) {
					var powers = {'k': 1, 'm': 2, 'g': 3, 't': 4};

					var regex = /(\d+(?:\.\d+)?)\s?(k|m|g|t)?b?/i;

					var res = regex.exec(value);

					if (!res[2] && jQuery.isNumeric(res[1])) {
						return res[1];
					}

					return res[1] * Math.pow(1024, powers[res[2].toLowerCase()]);
				}
			},
			/*
			 * Upload Size
			 */

			{
				id: 'typeup',
				label: 'File Mime Type',
				type: 'string',
				input: 'text',
				operators: ['equal', 'not_equal']

			},

			/*
			 * Subnet
			 */
			{
				id: 'subnet',
				label: 'Subnet',
				type: 'string',
				placeholder: '255.255.255.0/24',
				validation: {
					format: /^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])(\/(\d|[1-2]\d|3[0-2]))$/
				},
				operators: ['equal', 'not_equal']
			},

			/*
			 * Request Type
			 */
			{
				id: 'request',
				label: 'Request Type',
				type: 'string',
				input: 'select',
				values: {
					'upload': 'Upload',
					'get': 'GET',
					'post': 'POST',
					'put': 'PUT',
					'delete': 'DELETE',
					'public': 'Public Link'
				},
				operators: ['equal', 'not_equal']
			},

			/*
			 * Request URL
			 */
			{
				id: 'request-url',
				label: 'Request URL',
				type: 'string',
				placeholder: 'http://www.owncloud.org',
				validation: {
					rules: /(ftp|https?):\/\/[^ "]+$/
				},
				operators: ['equal', 'not_equal']
			},

			/*
			 * Time
			 */
			{
				id: 'time',
				label: 'Request Time',
				type: 'time',
				plugin: 'timepicker',
				plugin_config: {
					showTimezone: true,
					timeFormat: 'hh:mm tt z'
				},
				operators: ['equal', 'not_equal', 'less', 'less_or_equal', 'greater', 'greater_or_equal', 'between']
			},

			/*
			 * App
			 */
			{
				id: 'app',
				label: 'Server App',
				type: 'string',
				//input: 'select',
				input: function ($rule, filter) {
					var $input;

					// Very sloppy needs refactoring xxx
					if (!$rule.find('.rule-value-container select').length) {
						$input = $rule.find('.rule-value-container').append('<select name="selectApps"><option value="">--Loading--</option></select>').find('select');
					} else {
						$input = $rule.find('.rule-value-container select');
					}

					$.ajax({
						url: OC.generateUrl('settings/apps/list?category=0'),
						type: 'GET',
						dataType: 'json',
						localCache: true,
						cacheKey: 'appList'
					}).done(function (data) {

						var options = [];
						$input.empty();
						$.each(data.apps, function (index, value) {
							options.push('<option value="' + value.id + '">' + value.name + '</option>');
						});
						$input.append(options);

					});


				},
				onAfterSetValue: function ($rule, value) {
					var $input = $rule.find('.rule-value-container select');
					//$input.val(value).trigger('change');
					$input.ajaxSuccess(function () {
						$input.val(value).trigger('change');
					});
				},
				valueParser: function ($rule, value, filter, operator) {
					return $rule.find('[name=selectApps]').val();
				},
				operators: ['equal', 'not_equal']
			},

			/*
			 * Server App File
			 */
			{
				id: 'appFile',
				label: 'Server App File',
				type: 'string',
				operators: ['equal', 'not_equal']
			},

			/*
			 * User Agent
			 */
			{
				id: 'userAgent',
				label: 'User Agent',
				type: 'string',
				operators: ['equal', 'not_equal']

			},

			/*
			 * User Device Type
			 */
			{
				id: 'deviceType',
				label: 'User Device',
				type: 'string',
				input: 'select',
				values: {
					'android': 'Android',
					'ios': 'iOS',
					'desktop': 'Desktop',
					'webdav': 'WebDAV'
				},
				operators: ['equal', 'not_equal']
			},

			/*
			 * Logged In
			 * Value of Yes MUST be 1 in order for rule to function correctly
			 * object used to order correctly in ui and maintain proper value
			 */
			{
				id: 'loggedIn',
				label: 'User Logged In',
				type: 'string',
				input: 'radio',
				values: {
					1: 'Yes',
					0: 'No'
				},
				operators: ['equal']
			},

			/*
			 * User Group
			 * todo: add select 2
			 */
			{
				id: 'userGroup',
				label: 'User Group',
				type: 'string',
				input: function ($rule, filter) {

					var $input;
					// Very sloppy needs refactoring xxx
					if (!$rule.find('.rule-value-container select').length) {
						$input = $rule.find('.rule-value-container').append('<select name="selectGroup"><option value="">--Loading--</option></select>').find('select');
					} else {
						$input = $rule.find('.rule-value-container select');
					}

					$.ajax({
						type: 'GET',
						url: OC.generateUrl('/apps/firewall/ajax/getGroups'),
						dataType: 'json',
						localCache: true,
						cacheKey: 'groupList'
					}).done(function (data) {

						var options = [];
						$input.empty();
						$.each(data.data.adminGroups, function (index, value) {
							options.push('<option value="' + value.id + '">' + value.name + '</option>');
						});
						$.each(data.data.groups, function (index, value) {
							options.push('<option value="' + value.id + '">' + value.name + '</option>');

						});
						$input.append(options);
					});
				},
				valueParser: function ($rule, value, filter, operator) {
					return $rule.find('[name=selectGroup]').val();
				},
				onAfterSetValue: function ($rule, value) {
					var $input = $rule.find('.rule-value-container select');
					$input.val(value).trigger('change');

					$input.ajaxSuccess(function () {
						$input.val(value).trigger('change');
					});
				},
				operators: ['equal', 'not_equal']
			}
		]
	});

	// Get ui data and bind them to gui
	$.getJSON(OC.generateUrl('/apps/firewall/ajax/getUIData'), function (data) {
		var rules = $.parseJSON(data.rules);
		var debugLevel = $.parseJSON(data.debugLevel);

		// Add rules to GUI
		if (!jQuery.isEmptyObject(rules)) {
			$ruleBuilder.queryBuilder('setRules', rules);
		}

		// Set the debug level
		$('#firewallDebug').val(debugLevel);
	}).fail(function () {
		OC.Notification.show('Failed to fetch firewall rules.');
	});


	// Have to build regex select box down here after all other filters are declared
	function buildRegexSelect () {
		var filters = $ruleBuilder.queryBuilder('getFilters');
		var filterOptions = [];
		var unwantedFilters = ['regex', 'userTime', 'time'];
		var $list = $(filters);

		$list.each(function (index, value) {
			if (unwantedFilters.indexOf(value.id) === -1) {
				filterOptions.push('<option value="' + value.id + '">' + value.label + '</option>');
			}
		});

		$ruleBuilder.find('.regexRules').append(filterOptions);
	}

	/**
	 * Note: i hard coded the selector because I don't want this used by others.
	 * @param message
	 * @param type
	 */
	function firewallNotification (message, type) {
		type = type || 'success';

		$('#firewall .msg').text(message)
			.addClass(type)
			.stop(true, true)
			.delay(3000)
			.fadeOut(900)
			.show();
	}

	/*
	 * Event Handlers
	 */

	$('.saveRules').on('click', function (e) {
		e.preventDefault();

		$.ajax({
			type: "POST",
			dataType: "json",
			url: OC.generateUrl('/apps/firewall/ajax/save'),
			data: {rules: JSON.stringify($ruleBuilder.queryBuilder('getRules'))}
		}).done(function () {
			firewallNotification('Firewall rules saved.');
		}).fail(function () {
			firewallNotification('An error occurred while saving rules.', 'error');
		});
	});

	$('#firewallDebug').on('change', function (e) {
		$.ajax({
			type: "POST",
			dataType: "json",
			url: OC.generateUrl('/apps/firewall/ajax/debug'),
			data: {
				level: $(this).find('option:selected').val()
			}
		}).done(function () {
			firewallNotification('Firewall debug level saved.');
		}).fail(function () {
			firewallNotification('An error occurred while saving debug level.', 'error');
		});
	});
});
