<?php
/**
 * @author Clark Tomlinson
 * @copyright 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Firewall;


class Config extends \OC\Config {

	/**
	 * Use a different config file for the firewall
	 *
	 * @param string $configDir
	 */

	public function __construct($configDir) {
		parent::__construct($configDir, 'firewall.config.php');
		// We don't want to write debug info to the firewall config
		$this->debugMode = false;
		$this->configFilename = $this->configDir . 'firewall.config.php';
	}

	/**
	 * Get the list of firewall rules as a JSON string
	 *
	 * @param bool $decode
	 * @return string The rules as a JSON string
	 */
	public function getRules($decode = false) {
		$rules = $this->getValue('rules', '{}');
		/**
		 * We have to convert this to this format or our ui explodes
		 * todo: xxx fix this in x.0.1
		 */
		if ($rules === '[{}]') {
			$rules = '{}';
		}
		if ($decode) {
			return json_decode($rules);
		}
		return $rules;
	}

	/**
	 * Set the list of firewall rules as a JSON string
	 *
	 * @param string $rules The rules as a JSON string
	 */
	public function setRules($rules) {
		$this->setValue('rules', $rules);
	}


	/**
	 * Sets debug level to firewall config
	 *
	 * @param $level
	 */
	public function setDebug($level) {
		$this->setValue('debug', $level);
	}

	/**
	 * Returns current debug level in the config
	 *
	 * @return int
	 */
	public function getDebugLevel() {
		return $this->getValue('debug', 0);
	}

}
