<?php
/**
 * @author Clark Tomlinson
 * @copyright 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Firewall;

use OCA\Firewall\Events\Contracts\EventDispatcher;
use OCA\Firewall\Events\Event\RuleEvent;
use OCP\AppFramework\Http;
use OCP\IGroupManager;
use OCP\IRequest;
use OCP\IUserSession;

class Firewall {

	// Use the EventDispatcher trait to start up events
	use EventDispatcher;

	/**
	 * @var bool
	 */
	public $passed = false;
	/**
	 * @var Rules
	 */
	public $rules;
	/**
	 * @var IUserSession
	 */
	private $session;
	/**
	 * @var bool
	 */
	private $isAdmin;
	/**
	 * @var IRequest
	 */
	private $request;

	/**
	 * @param $rules
	 * @param array $listeners
	 * @param array $subscribers
	 * @param IUserSession $session
	 * @param IGroupManager $IsAdmin
	 * @param IRequest $request
	 */
	public function __construct($rules, array $listeners, array $subscribers, IUserSession $session, IGroupManager $IsAdmin, IRequest $request) {
		$this->rules = $rules;
		$this->addObservers($subscribers, $listeners);
		$this->session = $session;
		// Have to make sure user is logged in before we try to get their uid duh...
		if ($this->session->getUser()) {
			$this->isAdmin = $IsAdmin->isAdmin($this->session->getUser()->getUID());

		}
		$this->request = $request;
	}

	/**
	 * Check if the current request is allowed or not
	 */
	public function checkRequest() {

		// We don't want to block requests to login or admin users
		$user = $this->session->getUser();

		$logout = $this->request->getParam('logout', 'false');

		if (!$user || $this->isAdmin || $logout === 'true') {
			return $this->pass();
		}
		$ruleAssertions = $this->rules->check();

		if ($ruleAssertions->isResult()) {
			return $this->pass();
		} else {
			return $this->fail();
		}

	}

	/**
	 * Runs when a rule passes
	 *
	 * @return bool
	 */
	public function pass() {
		$this->dispatcher->dispatch('firewall.pass', new RuleEvent($this));
		$this->passed = true;
		return true;
	}

	/**
	 * Runs when a rule fail
	 *
	 * @return bool
	 */
	public function fail() {
		$this->dispatcher->dispatch('firewall.fail', new RuleEvent($this));
		return false;
	}
}
