<?php
/**
 * @author Clark Tomlinson  <clark@owncloud.com>
 * @since 1/20/15, 10:50 AM
 * @copyright 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 */

namespace OCA\Firewall\Migrator;


use Carbon\Carbon;

class RuleMigrator {

	/**
	 * @var array
	 */
	protected $stack = [];


	/**
	 * @param string $oldRules
	 * @return array
	 */
	public function migrateRules($oldRules) {
		$oldRules = json_decode($oldRules);

		$condition = new \stdClass();
		$condition->condition = end($oldRules)->operator;

		foreach ($oldRules as $legacy) {
			if ($legacy->type === 'condition') {
				$lookupValues = $this->mapRules($legacy->check);
				$rule = new \stdClass();
				$rule->id = $lookupValues['id'];
				$rule->field = $lookupValues['field'];
				$rule->type = $lookupValues['type'];
				$rule->operator = (isset($legacy->negate)) ? 'not_equal' : 'equal';
				$rule->value = (isset($lookupValues['value']) ? $lookupValues['value'] : $legacy->check);

				// Got to check for edge cases
				if ($legacy->check === 'regex') {
					$rule->value = "{$this->mapCritera($legacy->criteria)}||{$legacy->regex}";
				}

				if ($legacy->check === 'time' && isset($legacy->end)) {
					$rule->operator = 'between';
					$rule->value = [Carbon::createFromTime(explode(':', $legacy->begin)[0])->format(('g:i a O')), Carbon::createFromTime(explode(':', $legacy->end)[0])->format(('g:i a O'))];
				}

				$condition->rules[] = $rule;
			}
		}

		$this->stack[] = $condition;

		return json_encode($this->stack);
	}

	/**
	 * @param string $search
	 * @return array
	 */
	private function mapRules($search) {
		$lookup = [
			'cidr' => [
				'id' => 'cidr',
				'field' => 'cidr',
				'type' => 'sring',
			],
			'regex' => [
				'id' => 'regex',
				'field' => 'regex',
				'type' => 'string',
			],
			'subnet' => [
				'id' => 'subnet',
				'field' => 'subnet',
				'type' => 'string',
			],
			'verb' => [
				'id' => 'request',
				'field' => 'request',
				'type' => 'string',
			],
			'loggedin' => [
				'id' => 'loggedIn',
				'field' => 'loggedIn',
				'type' => '',
			],
			'usergroup' => [
				'id' => 'userGroup',
				'field' => 'userGroup',
				'type' => 'string',
			],
			'time' => [
				'id' => 'time',
				'field' => 'time',
				'type' => 'string',
			],
			'sizeup' => [
				'id' => 'sizeup',
				'field' => 'sizeUp',
				'type' => 'string',
			],
			'typeup' => [
				'id' => 'typeup',
				'field' => 'typeUp',
				'type' => 'string',
			],
			'upload' => [
				'id' => 'request',
				'field' => 'request',
				'type' => 'string',
				'value' => 'upload'
			]
		];
		return isset($lookup[$search]) ? $lookup[$search] : null;
	}

	/**
	 * @param string $search
	 * @return string
	 */
	private function mapCritera($search) {
		$critera = [
			'User-agent' => 'userAgent',
			'agent' => 'userAgent',
			'desktop' => 'deviceType',
			'ios' => 'deviceType',
			'android' => 'deviceType',
			'webdav' => 'request',
			'Public' => 'request',
			'Request-url' => 'request-uri',
			'request' => 'request',
			'Remote-addr' => 'cidr',
			'IP' => 'cidr',
			'time' => 'time',
			'Server-time' => 'server-time',
			'User-time' => 'user-time',
			'user' => 'user',
			'groups' => 'group',
			'app' => 'app',
			'appfile' => 'app'
		];
		return isset($critera[$search]) ? $critera[$search] : $search;
	}


}
