<?php

namespace OCA\Firewall\Rules;

use OCA\Firewall\Rules\Contracts\Rule;


/**
 * Class CIDR
 *
 * @package OCA\Firewall\Rules
 *
 * Sample rule - Requested URL must be within the 192.168.1.0 subnet:
 *
 * {
 *   "type": "condition",
 *   "check": "cidr",
 *   "cidr": "192.168.1.0/24"
 * }
 *
 */
class CIDR extends Rule {

	/**
	 * @param object $rule
	 * @return bool
	 */
	public function doCheck($rule) {
		return $this->cidrMatch($rule->value, $this->criteria['remote-addr']);
	}

	/**
	 * Matches a cidr to the requested ip
	 *
	 * @param string $range
	 * @param string $ip
	 * @return bool
	 */
	protected function cidrMatch($range, $ip) {
		list($subnet, $bits) = explode('/', $range);

		$ip = ip2long($ip);
		$subnet = ip2long($subnet);
		$mask = -1 << (32 - $bits);
		$subnet &= $mask; # nb: in case the supplied subnet wasn't correctly aligned
		return ($ip & $mask) == $subnet;
	}
}