<?php

namespace OCA\Firewall\Rules\Contracts;

use Hoa\Ruler\Context;
use Hoa\Ruler\Ruler;


/**
 * Class Rule
 *
 * @package OCA\Firewall\Rules
 * This is the abstract base class for creating new firewall rules.
 */
abstract class Rule {

	/** @var \OC_L10N $l l10n object for translations */
	protected $l;
	protected $criteria;
	protected $ruler;

	/**
	 * Class Constructor
	 *
	 * @param Ruler $ruler
	 * @param $critera
	 */
	public function __construct(Ruler $ruler, $critera) {
		$this->l = new \OC_L10N('firewall');
		$this->ruler = $ruler;
		$this->criteria = $critera;
	}

	/**
	 * Return whether the request passes the check or not
	 *
	 * @abstract
	 * @param object $rule rule object
	 * @return boolean True if the check passes, false if the check fails
	 */
	abstract public function doCheck($rule);

	/**
	 * Get the result of our rule's
	 *
	 * @param $rule
	 * @param $concreteValue
	 * @return bool
	 */
	final public function assert($rule, $concreteValue) {

		$context = new Context(array(
			'ruleVal' => $rule->value,
			'value' => $concreteValue
		));

		return $this->ruler->assert($this->evalRule($rule), $context);
	}

	/**
	 * @param $rule
	 * @return string
	 */
	private function evalRule($rule) {
		return 'ruleVal ' . $rule->operator . ' value';
	}


}