<?php

namespace OCA\Firewall\Rules;

use OCA\Firewall\Rules\Contracts\Rule;

/**
 * Class Regex
 *
 * @package OCA\Firewall\Rules
 *
 * Sample rule - Requested URL must contain the letter "p":
 *
 * {
 *   "type": "condition",
 *   "check": "regex",
 *   "regex": "#p#",
 *   "criteria": "request"
 * }
 *
 */
class Regex extends Rule {

	/**
	 * @param object $rule
	 * @return bool
	 */
	public function doCheck($rule) {
		$values = explode('||', $rule->value, 2);
		$criteria = $this->mapCriteria($this->criteria);

		// If we get an array from criteria (usergroups) merge them into a string to match
		if (is_array($criteria[$values[0]])) {
			$criteria[$values[0]] = implode(',', $criteria[$values[0]]);
		}
		$result = (bool)preg_match('/' . $values[1] . '/', $criteria[$values[0]]);
		if ($rule->operator === 'equal' || $rule->operator === '=') {
			return $result;
		}
		return !$result;
	}

	/**
	 * Map rule values to criteria names
	 *
	 * @param array $criteria
	 * @return array
	 */
	private function mapCriteria($criteria) {
		// Todo fix typeup criteria
		return array(
			'cidr' => $criteria['remote-addr'],
			'subnet' => $criteria['remote-addr'],
			'request' => $criteria['request'],
			'request-uri' => $criteria['request-uri'],
			'request-url' => $criteria['request-uri'],
			'time' => $criteria['time'],
			'app' => $criteria['app'],
			'appFile' => $criteria['request-uri'],
			'serverTime' => $criteria['server-time'],
			'userAgent' => $criteria['user-agent'],
			'deviceType' => $criteria['agent'],
			'userGroup' => $criteria['groups'],
			'userTime' => $criteria['user-time'],
			'typeup' => ''
		);
	}
}
