<?php
/**
 * @author Clark Tomlinson  <clark@owncloud.com>
 * @since 11/7/14, 9:34 AM
 * @copyright 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/
 */

namespace OCA\Firewall\Rules;


use OCA\Firewall\Rules\Contracts\Rule;

class Request extends Rule {

	/**
	 * Return whether the request passes the check or not
	 *
	 * @param object $rule rule object
	 * @return boolean True if the check passes, false if the check fails
	 */
	public function doCheck($rule) {
		if ($rule->value === 'upload') {
			return $this->checkUploadRequest($rule);
		} elseif ($rule->value === 'public') {
			return $this->checkPublicRequest($rule);
		}
		return $this->assert($rule, strtolower($this->criteria['method']));
	}

	/**
	 * Check for the presence of upload.php in the uri
	 *
	 * @param object $rule
	 * @return bool
	 */
	private function checkUploadRequest($rule) {
		// Since this method uses a custom operator we'll need to change it here.
		$this->modifyOperator($rule);
		return $this->assert($rule, $this->criteria['request']);
	}

	/**
	 * Check for the presence of public.php in the uri
	 * and make sure it is NOT the last part
	 *
	 * @param object $rule
	 * @return bool
	 */
	private function checkPublicRequest($rule) {
		// Since this method uses a custom operator we'll need to change it here.
		$this->modifyOperator($rule);
		return $this->assert($rule, $this->criteria['request']);
	}

	/**
	 * This method is purely to eliminate duplication for the checkPublicRequest
	 * and checkUploadRequest methods.
	 *
	 * @param $rule
	 */
	private function modifyOperator($rule) {
		if ($rule->operator === 'equal' || $rule->operator === '=') {
			$rule->operator = 'uri_contains';
		} else {
			$rule->operator = 'uri_not_contains';
		}
	}
}
