<?php

namespace OCA\Firewall\Rules;

use OCA\Firewall\Rules\Contracts\Rule;

/**
 * Class SizeUp
 *
 * @package OCA\Firewall\Rules
 *
 * Sample rule - Request must be no more than a specific size:
 *
 * {
 *   "type": "condition",
 *   "check": "sizeup",
 *   "limit": "1000000"
 * }
 *
 */
class SizeUp extends Rule {

	/**
	 * @param object $rule
	 * @return bool
	 */
	public function doCheck($rule) {

		/*
		 * Since we cant easily one off change the wording of the rule
		 * in the UI they make this very unclear for the end user so
		 * we have to flip them here
		 * todo: xxx HAS to be a better way to resolve this
		 */
		switch ($rule->operator) {
			case 'less':
				$rule->operator = 'greater';
				break;

			case 'less_or_equal':
				$rule->operator = 'greater_or_equal';
				break;

			case 'greater':
				$rule->operator = 'less';
				break;

			case 'greater_or_equal':
				$rule->operator = 'less_or_equal';
				break;
		}

		if (isset($_SERVER['HTTP_OC_TOTAL_LENGTH'])) {
			$size = intval($_SERVER['HTTP_OC_TOTAL_LENGTH']);
		} elseif (isset($_SERVER['CONTENT_LENGTH'])) {
			$size = intval($_SERVER['CONTENT_LENGTH']);
		} else {
			// If we cant find a size pass the rule
			return true;
		}

		return $this->assert($rule, $size);
	}
}
