<?php

namespace OCA\Firewall\Rules;

use Carbon\Carbon;
use Hoa\Ruler\Context;
use OCA\Firewall\Rules\Contracts\Rule;

/**
 * Class Time
 *
 * @package OCA\Firewall\Rules
 *
 * Sample rule - Request must occur between 9:00am and 5:00pm server time:
 *
 * {
 *   "type": "condition",
 *   "check": "time",
 *   "begin": "9:00",
 *   "end": "17:00"
 * }
 *
 */
class Time extends Rule {


	public function doCheck($rule) {
		// Server time is always UTC as seen in base.php:479
		$serverTime = Carbon::createFromTimestamp($this->criteria['time']);

		if ($rule->operator === 'between') {

			$time1 = Carbon::createFromFormat('g:i a O', $rule->value[0])->setTimezone('UTC');
			$time2 = Carbon::createFromFormat('g:i a O', $rule->value[1])->setTimezone('UTC');

			return $time1->toTimeString() <> $time2->toTimeString();
		}


		$rule->value = Carbon::createFromFormat('g:i a O', $rule->value)->setTimezone('UTC')->toTimeString();

		return $this->assert($rule, $serverTime->toTimeString());
	}
}