<?php
/**
 * @author Clark Tomlinson  <clark@owncloud.com>
 * @since 2/3/15, 1:30 PM
 * @copyright 2015 ownCloud, Inc.
 *
 * This file is licensed under the Affero General Public License version 3 or later.
 * See the COPYING-README file.
 */

namespace OCA\Firewall\Tests\Helpers;


use Hoa\Ruler\Ruler;
use OC\AppFramework\Http\Request;
use OCA\Firewall\Rules;
use OCP\IConfig;
use OCP\IRequest;
use OCP\Security\ISecureRandom;
use Test\TestCase;

class RuleTester extends TestCase {

	/**
	 * @var \Hoa\Ruler\Ruler
	 */
	protected $ruler;
	/**
	 * @var \OCA\Firewall\Rules\Contracts\Rule
	 */
	protected $rule;

	/**
	 * @var \stdClass
	 */
	protected $ruleMock;
	/**
	 * @var IRequest
	 */
	protected $requestMock;
	/**
	 * @var ISecureRandom
	 */
	protected $secureRandom;
	/**
	 * @var IConfig
	 */
	protected $config;

	/**
	 * Sets up mock and creates instance of rule named in $ruleInstance
	 * also sets default operator
	 */
	public function scaffold() {
		$this->ruler = new Ruler();
		$this->ruleMock = $this->getMockBuilder('Rule');
		$this->ruleMock->operator = '=';
		$this->config = $this->getMock('\OCP\IConfig');

		$this->secureRandom = $this->getMock('\OCP\Security\ISecureRandom');

		$vars = array(
			'server' => [
				'SCRIPT_NAME' => 'upload.php',
				'SCRIPT_FILENAME' => 'upload.php',
				'REQUEST_METHOD' => 'POST',

			],
			'request' => '/public.php/apps/files/upload.php',
		);

		$this->requestMock = new Request($vars, $this->secureRandom, $this->config);
		/**
		 * todo: Remove this once Rules class is tested
		 * these are here only to add custom operators for other tests to use.
		 */
		$rules = new Rules(
			$this->ruleMock,
			$this->ruler,
			$this->getMock('\Mobile_Detect'),
			$this->requestMock
		);
		\Test_Helper::invokePrivate($rules, 'addOperators');
	}

	/**
	 *
	 */
	public function tearDown() {
	}

}
