<?php
/**
 * @author Clark Tomlinson  <clark@owncloud.com>
 *
 * @copyright Copyright (c) 2015, ownCloud, Inc.
 * @license AGPL-3.0
 */


namespace OCA\Firewall\Tests\Lib;


use OCA\Firewall\Firewall;
use Test\TestCase;

class FirewallTest extends TestCase {
	/**
	 * @var \PHPUnit_Framework_MockObject_MockObject
	 */
	private $userSessionMock;
	/**
	 * @var \PHPUnit_Framework_MockObject_MockObject
	 */
	private $requestMock;
	/**
	 * @var \PHPUnit_Framework_MockObject_MockObject
	 */
	private $rulesMock;
	/**
	 * @var Firewall
	 */
	private $instance;

	public function testCheckRequest() {
		$this->userSessionMock->expects($this->exactly(2))
			->method('getUser')
			->willReturn(true);

		$this->requestMock->expects($this->exactly(2))
			->method('getParam')
			->with('logout', 'false')
			->willReturnOnConsecutiveCalls(false, 'true');

		$this->rulesMock->expects($this->once())
			->method('check')
			->willReturnSelf();

		$this->assertFalse($this->instance->checkRequest());

		$this->assertTrue($this->instance->checkRequest());

	}

	public function testPass() {
		$this->assertTrue($this->instance->pass());
	}

	public function testFail() {
		$this->assertFalse($this->instance->fail());
	}

	protected function setUp() {

		$this->rulesMock = $this->getMockBuilder('OCA\Firewall\Rules')
			->disableOriginalConstructor()
			->getMock();

		$listeners = [];
		$subscribers = [];
		$this->userSessionMock = $this->getMockBuilder('OCP\IUserSession')
			->disableOriginalConstructor()
			->getMock();

		$groupManagerMock = $this->getMockBuilder('OCP\IGroupManager')
			->disableOriginalConstructor()
			->getMock();

		$this->requestMock = $this->getMockBuilder('OCP\IRequest')
			->disableOriginalConstructor()
			->getMock();

		$this->instance = new Firewall($this->rulesMock,
			$listeners,
			$subscribers,
			$this->userSessionMock,
			$groupManagerMock,
			$this->requestMock);
		parent::setUp();

	}

}
