<?php
/**
 * @author Clark Tomlinson  <clark@owncloud.com>
 *
 * @copyright Copyright (c) 2015, ownCloud, Inc.
 * @license AGPL-3.0
 */


namespace OCA\Firewall\Tests\Lib;


use OCA\Firewall\Rules;
use Test\TestCase;

class RulesTest extends TestCase {
	/**
	 * @var Rules
	 */
	private $instance;

	/**
	 * @var string
	 */
	private $dummyrules = '[{"name":"","condition":"AND","rules":[{"id":"typeup","field":"typeup","type":"string","input":"text","operator":"equal","value":"message/rfc822"}]}]';

	/**
	 *
	 */
	public function testGetCriteria() {
		$this->assertEquals(
			[
				'user-agent' => 1,
				'agent' => 'android',
				'request-uri' => '',
				'request' => '',
				'remote-addr' => '',
				'ip' => '',
				'method' => '',
				'time' => time(),
				'user' => '',
				'groups' =>
					[]
				,
				'app' => ''
			],
			\Test_Helper::invokePrivate($this->instance, 'getCriteria'));
	}

	/**
	 *
	 */
	public function testBuild() {
		$this->assertFalse($this->instance->build($this->dummyrules));
	}

	/**
	 *
	 */
	public function testLoop() {

		$this->assertEmpty($this->instance->getStack());
		\Test_Helper::invokePrivate($this->instance,
			'loop',
			[json_decode($this->dummyrules), 1]);

		$this->assertEquals(
			[
				'1' => [
					'groupName' => '',
					'condition' => 'AND',
					'rules' =>
						[
							1
						]
				]
			],
			$this->instance->getStack());

	}

	/**
	 *
	 */
	public function testEvalCondition() {

		$dummyRule = new \StdClass();
		$dummyRule->id = 'typeup';
		$dummyRule->field = 'typeup';
		$dummyRule->type = 'string';
		$dummyRule->operator = 'equal';
		$dummyRule->value = 'message/rfc/822';


		$this->assertTrue(\Test_Helper::invokePrivate($this->instance,
			'evalCondition',
			[$dummyRule]));
	}

	/**
	 *
	 */
	public function testSetResult() {
		\Test_Helper::invokePrivate($this->instance, 'setResult', ['banana']);
		$this->assertEquals('banana', $this->instance->isResult());
	}

	/**
	 *
	 */
	public function testGetStack() {
		$this->assertEquals([], $this->instance->getStack());
	}

	/**
	 *
	 */
	public function testGetRules() {
		$this->assertFalse($this->instance->getRules());
	}


	/**
	 *
	 */
	protected function setUp() {
		parent::setUp();
		$irequestMock = $this->getMockBuilder('OCP\IRequest')
			->disableOriginalConstructor()
			->getMock();

		$irequestMock->server = ['REQUEST_METHOD' => ''];

		$rulerMock = $this->getMockBuilder('Hoa\Ruler\Ruler')
			->setMethods([
				'getAsserter'
			])
			->getMock();

		$asserterMock = $this->getMockBuilder('\Hoa\Ruler\Visitor\Asserter')
			->disableOriginalConstructor()
			->getMock();
		$rulerMock->expects($this->atLeastOnce())
			->method('getAsserter')
			->willReturn($asserterMock);

		$asserterMock->expects($this->exactly(19))
			->method('setOperator')
			->willReturn(true);

		$asserterMock->expects($this->exactly(6))
			->method('getOperator')
			->willReturn(true);

		$mobileDetectMock = $this->getMockBuilder('\Mobile_Detect')
			->disableOriginalConstructor()
			->getMock();

		$mobileDetectMock->expects($this->any())
			->method($this->anything())
			->willReturn(true);

		$this->instance = new Rules($this->dummyrules,
			$rulerMock,
			$mobileDetectMock,
			$irequestMock);

	}

	protected function tearDown() {
		parent::tearDown();
	}
}
