<?php
/**
 * @author Clark Tomlinson  <clark@owncloud.com>
 * @since 2/2/15, 2:48 PM
 * @copyright 2015 ownCloud, Inc.
 *
 * This file is licensed under the Affero General Public License version 3 or later.
 * See the COPYING-README file.
 */

namespace OCA\Firewall\Tests\Rules;


use OCA\Firewall\Rules\SizeUp;
use OCA\Firewall\Tests\Helpers\RuleTester;

class SizeUpTest extends RuleTester {

	public function setUp() {
		parent::scaffold();

		// Scaffold and create mocks
		$this->rule = new SizeUp($this->ruler, []);
	}

	public function testDoCheckWithContentLength() {
		$_SERVER['CONTENT_LENGTH'] = 10485760;

		$this->ruleMock->value = '10485760';

		$this->assertTrue($this->rule->doCheck($this->ruleMock));
	}

	public function testDoCheckWithOCLength() {
		$_SERVER['HTTP_OC_TOTAL_LENGTH'] = 10485760;


		$this->ruleMock->value = '10485760';


		$this->assertTrue($this->rule->doCheck($this->ruleMock));
	}

	public function testDoCheckWithoutLength() {

		$this->ruleMock->value = '10485760';


		$this->assertTrue($this->rule->doCheck($this->ruleMock));
	}

	public function testDoCheckWithBothHeadersWillChooseTotalLength() {
		$_SERVER['HTTP_OC_TOTAL_LENGTH'] = 10485760;
		$_SERVER['CONTENT_LENGTH'] = 10;


		$this->ruleMock->value = '10485760';


		$this->assertTrue($this->rule->doCheck($this->ruleMock));
	}

	public function testTotalLengthWillFailWithWrongSize() {

		$_SERVER['HTTP_OC_TOTAL_LENGTH'] = 10485;

		$this->ruleMock->value = '10485760';

		$this->assertFalse($this->rule->doCheck($this->ruleMock));
	}

	public function testContentLengthWillFailWithWrongSize() {

		$_SERVER['CONTENT_LENGTH'] = 10;

		$this->ruleMock->value = '10485760';

		$this->assertFalse($this->rule->doCheck($this->ruleMock));
	}

}
