<?php
/**
 * ownCloud
 *
 * @author Jesus Macias Portela <jesus@owncloud.com>
 * @author Juan Pablo Villafañez Ramos <jvillafanez@owncloud.com>
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */


namespace OCA\sharepoint\tests;

use OCA\sharepoint\lib\ConfigMgmt;

if (!class_exists('Crypt_Rijndael', false)) {
    include('Crypt/Rijndael.php');
}

class configMgmtTest extends \PHPUnit_Framework_TestCase {

    private $config;
    private $configMgmt;

    public function setUp(){
        $this->config = include('sharepoint/tests/config.php');

        \OC_User::createUser($this->config['global']['testUser'], $this->config['global']['testUserPassword']);
        \OC_User::setUserId($this->config['global']['testUser']);
        //mkdir(\OC_User::getHome($this->config['global']['testUser']));

        \OC_User::createUser('testUser2', 'testUser2');
        \OC_User::setUserId('testUser2');

        \OC_User::createUser('testUser3', 'testUser3');
        \OC_User::setUserId('testUser3');

        $this->configMgmt = New ConfigMgmt();
    }

    public function tearDown(){
        \OC_User::deleteUser($this->config['global']['testUser']);
        \OC_User::deleteUser('testUser2');
        \OC_User::deleteUser('testUser3');
    }

    public function test_create_global_mount() {
        $this->assertTrue($this->configMgmt->create_global_mount($this->config['configMgmt']['globalMountPoint'],
                                                                 $this->config['configMgmt']['applicables'],
                                                                 $this->config['configMgmt']['siteURL'],
                                                                 $this->config['configMgmt']['listName'],
                                                                 '1',
                                                                 null,
                                                                 null
                                                                 ));
    }

    public function test_create_same_global_mount() {
        $this->assertFalse($this->configMgmt->create_global_mount($this->config['configMgmt']['globalMountPoint'],
                                                                 $this->config['configMgmt']['applicables'],
                                                                 $this->config['configMgmt']['siteURL'],
                                                                 $this->config['configMgmt']['listName'],
                                                                 '1',
                                                                 null,
                                                                 null
                                                                 ));
    }

    public function test_update_global_mount() {
        $mount = $this->find_global_mount_by_name($this->config['configMgmt']['globalMountPoint']);
        $this->assertTrue($this->configMgmt->update_global_mount($mount['mount_id'],
                                                                 $this->config['sharepoint2010']['site']['user'],
                                                                 $this->config['sharepoint2010']['site']['password'],
                                                                 '3'
                                                                 ));
        $mount = $this->find_global_mount_by_name($this->config['configMgmt']['globalMountPoint']);
        $this->assertNotNull($mount['user']);
        $this->assertNotNull($mount['password']);
        $this->AssertEquals($mount['auth_type'], '3');
        $this->assertTrue($this->configMgmt->update_global_mount($mount['mount_id'],
                                                                 $this->config['sharepoint2010']['site']['user'],
                                                                 $this->config['sharepoint2010']['site']['password'],
                                                                 '2'
                                                                 ));
    }

    public function test_save_custom_credentials() {
        $mount = $this->find_global_mount_by_name($this->config['configMgmt']['globalMountPoint']);
        $this->assertTrue($this->configMgmt->save_custom_credentials($mount['mount_id'],
                                                                 $this->config['global']['testUser'],
                                                                 $this->config['sharepoint2010']['site']['user'],
                                                                 $this->configMgmt->encryptPassword($this->config['sharepoint2010']['site']['password'])
                                                                 ));
        $this->assertTrue($this->configMgmt->save_custom_credentials($mount['mount_id'],
                                                                 $this->config['global']['testUser'],
                                                                 $this->config['sharepoint2010']['site']['user'],
                                                                 $this->configMgmt->encryptPassword($this->config['sharepoint2010']['site']['password'])
                                                                 ));
    }

    public function test_get_custom_credentials() {
        $credentials = array();
        $mount = $this->find_global_mount_by_name($this->config['configMgmt']['globalMountPoint']);
        $cursor = $this->configMgmt->get_custom_credentials($mount['mount_id'], $this->config['global']['testUser']);
        if($row = $cursor->fetchRow()){
            $credentials['user'] = $row['credential_username'];
            if($row['credential_password'] !== '' ){
                $credentials['password'] = $this->configMgmt->decryptPassword($row['credential_password']);
            }
        }
        $this->AssertEquals($this->config['sharepoint2010']['site']['user'], $credentials['user']);
        $this->AssertEquals($this->config['sharepoint2010']['site']['password'], $credentials['password']);
    }

    public function test_create_user_mount() {
        $this->assertTrue($this->configMgmt->create_personal_mount($this->config['global']['testUser'],
                                                               $this->config['configMgmt']['userMountPoint'],
                                                               $this->config['configMgmt']['siteURL'],
                                                               $this->config['configMgmt']['listName'],
                                                               null,
                                                               null,
                                                               2));
    }

    public function test_create_same_user_mount() {
        $this->assertFalse($this->configMgmt->create_personal_mount($this->config['global']['testUser'],
                                                               $this->config['configMgmt']['userMountPoint'],
                                                               $this->config['configMgmt']['siteURL'],
                                                               $this->config['configMgmt']['listName'],
                                                               null,
                                                               null,
                                                               2));
    }

    public function test_get_global_mounts() {
        $mounts = $this->configMgmt->get_global_mounts();
        foreach($mounts as $mount) {
            if($mount['mount_point'] === $this->config['configMgmt']['globalMountPoint']){
                $this->AssertEquals($mount['url'], $this->config['configMgmt']['siteURL']);
                $this->AssertEquals($mount['list_name'], $this->config['configMgmt']['listName']);
            }
        }
    }

    public function test_get_global_user_mounts() {
        $cursor = $this->configMgmt->get_mounts_for_user($this->config['global']['testUser'], array($this->config['global']['testGroup']));
        if ($cursor !== false) {
            $mounts = array();
            while(($row = $cursor->fetchRow()) !== false) {
                $mounts[] = $row;
            }
            $this->AssertEquals(count($mounts), 1);
            $this->AssertEquals($mounts[0]['user'], null);
            $this->AssertEquals($mounts[0]['mount_point'], $this->config['configMgmt']['globalMountPoint']);
            $this->AssertEquals($mounts[0]['url'], $this->config['configMgmt']['siteURL']);
            $this->AssertEquals($mounts[0]['list_name'], $this->config['configMgmt']['listName']);
        }
    }

    public function test_getAllMounts() {
        $cursor = $this->configMgmt->getAllMounts();
        $i=0;
        if ($cursor !== false) {
            while (($row = $cursor->fetchRow()) !== false) {
                $i++;
            }
        }
        $this->AssertEquals($i, 1) ;
    }

    public function test_getAllMountsWithCredentials() {
        $cursor = $this->configMgmt->getAllMountsWithCredentials();
        if ($cursor !== false) {
            while (($row = $cursor->fetchRow()) !== false) {
                $this->assertEquals($row['credential_username'], $this->config['sharepoint2010']['site']['user']) ;
                $this->assertArrayHasKey('password', $row) ;
            }
        }
    }

    public function test_getAllPersonalMounts() {
        $cursor = $this->configMgmt->getAllPersonalMounts();
        $i=0;
        if ($cursor !== false) {
            while (($row = $cursor->fetchRow()) !== false) {
                $i++;
            }
        }
        $this->AssertEquals($i, 1) ;
    }

    public function test_get_personal_user_mounts() {
        $cursor = $this->configMgmt->get_personal_mounts_per_user($this->config['global']['testUser']);
        if ($cursor !== false) {
            $mounts = array();
            while(($row = $cursor->fetchRow()) !== false) {
                $mounts[] = $row;
            }
            $this->AssertEquals(count($mounts), 1);
            $this->AssertEquals($mounts[0]['user'], null);
            $this->AssertEquals($mounts[0]['mount_point'], $this->config['configMgmt']['userMountPoint']);
            $this->AssertEquals($mounts[0]['url'], $this->config['configMgmt']['siteURL']);
            $this->AssertEquals($mounts[0]['list_name'], $this->config['configMgmt']['listName']);
        }
    }

    public function test_set_user_credentials() {
        $this->AssertTrue($this->configMgmt->set_user_credentials($this->config['global']['testUser'],
                                                $this->config['sharepoint2010']['site']['user'],
                                                $this->config['sharepoint2010']['site']['password']));

        $this->AssertTrue($this->configMgmt->set_user_credentials($this->config['global']['testUser'],
                                                $this->config['sharepoint2010']['site']['user'],
                                                null));

        $this->AssertTrue($this->configMgmt->set_user_credentials('testUser2',
                                                'user',
                                                null));

        $this->AssertTrue($this->configMgmt->set_user_credentials('testUser3',
                                                'user',
                                                'password'));
    }

    public function test_get_password_for_user_not_exists() {
        $this->AssertFalse($this->configMgmt->get_password_for_user("johndoe"));
    }


    public function test_get_password_for_user() {
        $credentials = $this->configMgmt->get_password_for_user($this->config['global']['testUser']);
        $this->AssertEquals($credentials[0], $this->config['sharepoint2010']['site']['user']);
        $this->AssertEquals($credentials[1], $this->config['sharepoint2010']['site']['password']);
    }

    public function test_delete_user_mount() {
        $this->assertTrue($this->configMgmt->delete_user_mount($this->config['configMgmt']['userMountPoint'],
                                                               $this->config['global']['testUser']));
    }

    public function test_delete_global_mount() {
        $mount = $this->find_global_mount_by_name($this->config['configMgmt']['globalMountPoint']);
        $this->assertTrue($this->configMgmt->delete_global_mount($mount['mount_id']));
    }

    //Auth3 -> Custom credentials
    public function test_create_global_mount_auth3() {
        $this->assertTrue($this->configMgmt->create_global_mount($this->config['configMgmt']['globalMountPoint'],
                                                                 $this->config['configMgmt']['applicables'],
                                                                 $this->config['configMgmt']['siteURL'],
                                                                 $this->config['configMgmt']['listName'],
                                                                 '3',
                                                                 'test',
                                                                 'test'
                                                                 ));

        $mount = $this->find_global_mount_by_name($this->config['configMgmt']['globalMountPoint']);
        $this->assertTrue($mount['user']!==null);
        $this->assertTrue($mount['password']!==null);
        $this->assertTrue($this->configMgmt->delete_global_mount($mount['mount_id']));
    }

     public function test_create_personal_mount_auth1() {
        $this->assertTrue($this->configMgmt->create_personal_mount($this->config['global']['testUser'],
                                                                    $this->config['configMgmt']['userMountPoint'],
                                                                    $this->config['configMgmt']['siteURL'],
                                                                    $this->config['configMgmt']['listName'],
                                                                    $this->config['sharepoint2010']['site']['user'],
                                                                    $this->config['sharepoint2010']['site']['password'],
                                                                    1));
        $mount = $this->find_personal_mount_by_name($this->config['configMgmt']['userMountPoint']);
        $this->assertTrue($mount['user']!==null);
        $this->assertTrue($mount['password']!==null);
        $this->assertTrue($this->configMgmt->delete_user_mount($this->config['configMgmt']['userMountPoint'],
                                                                $this->config['global']['testUser']));
    }

    public function test_getPersonalMountByIdAndUser() {
        $this->assertTrue($this->configMgmt->create_personal_mount($this->config['global']['testUser'],
                                                                    $this->config['configMgmt']['userMountPoint'],
                                                                    $this->config['configMgmt']['siteURL'],
                                                                    $this->config['configMgmt']['listName'],
                                                                    $this->config['sharepoint2010']['site']['user'],
                                                                    $this->config['sharepoint2010']['site']['password'],
                                                                    1));

        $mount = $this->find_personal_mount_by_name($this->config['configMgmt']['userMountPoint']);
        $cursor = $this->configMgmt->getPersonalMountByIdAndUser($mount['mount_id'],
                                                            $this->config['global']['testUser']);
        if ($cursor !== false) {
            $mountb = $cursor->fetchRow();
            $this->assertEquals($mountb['mount_point'], $this->config['configMgmt']['userMountPoint']);
            $this->assertTrue($this->configMgmt->deletePersonalMountByIdAndUser($mount['mount_id'],
                                                                    $this->config['global']['testUser']));
        }
    }

    public function test_updatePersonalMountCredentials() {
        $this->assertTrue($this->configMgmt->create_personal_mount($this->config['global']['testUser'],
                                                                    $this->config['configMgmt']['userMountPoint'],
                                                                    $this->config['configMgmt']['siteURL'],
                                                                    $this->config['configMgmt']['listName'],
                                                                    $this->config['sharepoint2010']['site']['user'],
                                                                    $this->config['sharepoint2010']['site']['password'],
                                                                    1));
        $mount = $this->find_personal_mount_by_name($this->config['configMgmt']['userMountPoint']);
        $this->assertTrue($this->configMgmt->updatePersonalMountCredentials($mount['mount_id'], 'user', 'password', 2));
        $this->assertTrue($this->configMgmt->updatePersonalMountCredentials($mount['mount_id'], $this->config['sharepoint2010']['site']['user'], $this->config['sharepoint2010']['site']['password'], 1));
        $this->assertTrue($this->configMgmt->deletePersonalMountByIdAndUser($mount['mount_id'],
                                                                    $this->config['global']['testUser']));
    }

    public function test_getMountById() {
        $this->assertTrue($this->configMgmt->create_global_mount($this->config['configMgmt']['globalMountPoint'],
                                                                 $this->config['configMgmt']['applicables'],
                                                                 $this->config['configMgmt']['siteURL'],
                                                                 $this->config['configMgmt']['listName'],
                                                                 '1',
                                                                 null,
                                                                 null
                                                                 ));
        $mount = $this->find_global_mount_by_name($this->config['configMgmt']['globalMountPoint']);
        $cursor = $this->configMgmt->getMountById($mount['mount_id']);
        if ($cursor !== false) {
            $mountb = $cursor->fetchRow();
            $this->assertEquals($mountb['mount_point'], $this->config['configMgmt']['globalMountPoint']);
            $this->assertTrue($this->configMgmt->delete_global_mount($mount['mount_id']));
        }
    }

    public function test_getAdminGlobalCredentialsCached() {
        $this->configMgmt->set_user_credentials('$harepointAdmin', $this->config['sharepoint2010']['site']['user'], $this->config['sharepoint2010']['site']['password']);
        $credentials = $this->configMgmt->getAdminGlobalCredentialsCached();
        $this->assertEquals($credentials['user'], $this->config['sharepoint2010']['site']['user']);
    }

    public function test_getUserGlobalCredentials() {
        $cursor = $this->configMgmt->getUserGlobalCredentials($this->config['global']['testUser']);
        if ($cursor !== false) {
            $credentials = $cursor->fetchRow();
            $this->assertEquals($credentials['user'], $this->config['sharepoint2010']['site']['user']);
        }
    }

    public function find_global_mount_by_name($name){
        $gmounts = $this->configMgmt->get_global_mounts();
        foreach ($gmounts as $mount) {
            if($mount['mount_point'] === $name){
                return $mount;
            }
        }
    }

    public function find_personal_mount_by_name($name){
        $pmounts = $this->configMgmt->get_personal_mounts_per_user($this->config['global']['testUser']);
        if ($pmounts !== false) {
            while(($row = $pmounts->fetchRow()) !== false) {
                if($row['mount_point'] === $name){
                    return $row;
                }
            }
        }
    }
}