<?php
/**
 * ownCloud
 *
 * @author Jesus Macias Portela <jesus@owncloud.com>
 * @author Juan Pablo Villafañez Ramos <jvillafanez@owncloud.com>
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */


namespace OCA\sharepoint\tests;

use OCA\sharepoint\lib\SHAREPOINT;

class sharepointFS_siteTest extends \PHPUnit_Framework_TestCase {

    private $config;
    private $client;

    protected static $random;

    public static function setUpBeforeClass(){
        self::$random = self::getRandomCode(8);
    }

    public function setUp(){

        if(!isset(self::$random)){
            self::$random = $this->getRandomCode();
        }
        $this->config = include('sharepoint/tests/config.php');

        \OC_User::deleteUser($this->config['global']['testUser']);
        \OC_User::createUser($this->config['global']['testUser'], $this->config['global']['testUserPassword']);
        \OC_User::setUserId($this->config['global']['testUser']);
        mkdir(\OC_User::getHome($this->config['global']['testUser']));

        $this->sharepoint = New SHAREPOINT(array(
                                                'apiurl'=>$this->config['sharepoint2010']['site']['siteURL'],
                                                'user'=>$this->config['sharepoint2010']['site']['user'],
                                                'password'=>$this->config['sharepoint2010']['site']['password'],
                                                'listName'=>$this->config['sharepoint2010']['site']['documentList'],
                                                'mountPoint'=>$this->config['sharepoint2010']['site']['mountPoint']));
    }

    public function tearDown(){
        \OC_User::deleteUser($this->config['global']['testUser']);
    }

    /**
     * @large
     */
    public function test_getId() {
        $id = 'sharepoint::' .$this->config['sharepoint2010']['site']['user']. '@' .$this->config['sharepoint2010']['site']['siteURL']. '/' .$this->config['sharepoint2010']['site']['documentList']. '::' .$this->config['sharepoint2010']['site']['mountPoint'];
        $this->assertEquals($this->sharepoint->getId(), $id);
    }

    /**
     * @large
     */
    public function test_mkdir() {
        $this->assertTrue($this->sharepoint->mkdir('unittest'.self::$random));
    }

    /**
     * @large
     */
    public function test_mkdir_same_dir() {
        $this->assertFalse($this->sharepoint->mkdir('unittest'.self::$random));
    }

    /**
     * @large
     */
    public function test_mkdir_bad_dir() {
        $this->assertFalse($this->sharepoint->mkdir('unittest'.self::$random.'/aaaa/bbbb'));
    }

    /**
     * @large
     */
    public function test_hasUpdated() {
        $this->assertTrue($this->sharepoint->hasUpdated('unittest'.self::$random, 0));
    }

    /**
     * @large
     */
    public function test_stat() {
        $response = $this->sharepoint->stat('unittest'.self::$random);
        $this->assertArrayHasKey('size', $response);
        $this->assertArrayHasKey('atime', $response);
        $this->assertArrayHasKey('mtime', $response);
        $this->assertArrayHasKey('ctime', $response);
    }

    /**
     * @large
     */
    public function test_stat_bad_path() {
        $this->assertFalse($this->sharepoint->stat('unittest'.self::$random.'/non.txt'));
    }

    /**
     * @large
     */
    public function test_filetype_dir() {
        $this->assertEquals('dir', $this->sharepoint->filetype('unittest'.self::$random));
    }

    /**
     * @large
     */
    public function test_isReadable() {
        $this->assertTrue($this->sharepoint->isReadable('unittest'.self::$random));
        $this->assertFalse($this->sharepoint->isReadable('unittest_fake'));
    }

    /**
     * @large
     */
    public function test_isUpdatable() {
        $this->assertTrue($this->sharepoint->isUpdatable('unittest'.self::$random));
    }

    /**
     * @large
     */
    public function test_isCreatable() {
        $this->assertTrue($this->sharepoint->isCreatable('unittest'.self::$random));
    }

    /**
     * @large
     */
    public function test_isDeletable() {
        $this->assertTrue($this->sharepoint->isDeletable('unittest'.self::$random));
    }

    /**
     * @large
     */
    public function test_getMimeType_dir() {
        $this->assertEquals('httpd/unix-directory', $this->sharepoint->getMimeType('unittest'.self::$random));
    }

    /**
     * @large
     */
    public function test_fopen_write() {
        $tmpfile = $this->sharepoint->fopen('unittest'.self::$random.'/test_file.txt' , 'c+');
        $this->assertEquals('resource', gettype($tmpfile));
        fwrite($tmpfile, 'writing to file');
        fclose($tmpfile);
        $this->assertTrue($this->sharepoint->file_exists('unittest'.self::$random.'/test_file.txt'));
    }

    /**
     * @large
     */
    public function test_fopen_read() {
        $tmpfile = $this->sharepoint->fopen('unittest'.self::$random.'/test_file.txt' , 'rb');
        $this->assertEquals('resource', gettype($tmpfile));
        $content = fgets($tmpfile, 4096);
        fclose($tmpfile);
        $this->assertEquals('writing to file', $content);
    }

    /**
     * @large
     */
    public function test_filetype_file() {
        $this->assertEquals('file', $this->sharepoint->filetype('unittest'.self::$random.'/test_file.txt'));
    }

    /**
     * @large
     */
    public function test_file_exists() {
        $this->assertTrue($this->sharepoint->file_exists('unittest'.self::$random.'/test_file.txt'));
    }

    /**
     * @large
     */
    public function test_file_no_exists() {
        $this->assertFalse($this->sharepoint->file_exists('unittest'.self::$random.'/test_no_file.txt'));
    }

    /**
     * @large
     */
    public function test_getMimeType_file() {
        $this->assertEquals('text/plain', $this->sharepoint->getMimeType('unittest'.self::$random.'/test_file.txt'));
    }

    /**
     * @large
     */
    public function test_rename_dir() {
        $this->assertTrue($this->sharepoint->rename('unittest'.self::$random, 'unittest_renamed'));
        $this->assertTrue($this->sharepoint->rename('unittest_renamed', 'unittest'.self::$random));
    }

    /**
     * @large
     */
    public function test_rename_file() {
        $this->assertTrue($this->sharepoint->rename('unittest'.self::$random.'/test_file.txt', 'unittest'.self::$random.'/renamed.txt'));
        $this->assertTrue($this->sharepoint->rename('unittest'.self::$random.'/renamed.txt', 'unittest'.self::$random.'/test_file.txt'));
    }

    /**
     * @large
     */
    public function test_touch() {
        $this->assertTrue($this->sharepoint->touch('unittest'.self::$random.'/test_touch_file.txt'));
        $this->assertFalse($this->sharepoint->touch('unittest'.self::$random.'/test_touch_file.txt'));
    }

    /**
     * @large
     */
    public function test_opendir() {
        $this->assertEquals('resource', gettype($this->sharepoint->opendir('unittest'.self::$random)));
    }

    /**
     * @large
     */
    public function test_rmdir() {
        $this->assertTrue($this->sharepoint->rmdir('unittest'.self::$random));
    }

    /**
     * @large
     */
    public function test_rmdir_non_exists_dir() {
        //Must check, because must return false
        $this->assertTrue($this->sharepoint->rmdir('unittest'.self::$random));
    }

    public static function getRandomCode($size){
        $an = 'abcdefghijklmnopqrstuvwxyz0123456789';
        $su = strlen($an) - 1;
        $result = "";
        for($i=0; $i<$size; $i++){
            $result .= substr($an, rand(0, $su), 1);
        }
        return $result;
    }

}