<?php

/**
 * ownCloud
 *
 * @author Jesus Macias Portela <jesus@owncloud.com>
 * @author Juan Pablo Villafañez Ramos <jvillafanez@owncloud.com>
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */


namespace OCA\sharepoint\tests;
use \OCA\sharepoint\lib\Utils;
use OCA\sharepoint\lib\ConfigMgmt;
use \OCA\sharepoint\lib\UserHooks;


class hooksTest extends \PHPUnit_Framework_TestCase {

    private $config;
    private $configMgmt;

    public function setUp(){
        $this->config = include('sharepoint/tests/config.php');
        \OC_User::createUser($this->config['global']['testUser'], $this->config['global']['testUserPassword']);
        \OC_User::setUserId($this->config['global']['testUser']);
        $this->configMgmt = New ConfigMgmt();
    }

    public function tearDown(){
        \OC_User::deleteUser($this->config['global']['testUser']);

    }

    public function test_mount_storage() {
        $this->configMgmt->create_personal_mount($this->config['global']['testUser'],
                                                                    $this->config['configMgmt']['userMountPoint'],
                                                                    $this->config['configMgmt']['siteURL'],
                                                                    $this->config['configMgmt']['listName'],
                                                                    $this->config['sharepoint2010']['site']['user'],
                                                                    $this->configMgmt->encryptPassword($this->config['sharepoint2010']['site']['password']),
                                                                    1);

        $this->configMgmt->create_personal_mount($this->config['global']['testUser'],
                                                                    $this->config['configMgmt']['userMountPoint'].'2',
                                                                    $this->config['configMgmt']['siteURL'],
                                                                    $this->config['configMgmt']['listName'],
                                                                    $this->config['sharepoint2010']['site']['user'],
                                                                    $this->configMgmt->encryptPassword($this->config['sharepoint2010']['site']['password']),
                                                                    2);

        $this->configMgmt->set_user_credentials($this->config['global']['testUser'], 'user', 'password');

        $this->configMgmt->create_global_mount($this->config['configMgmt']['globalMountPoint'],
                                             $this->config['configMgmt']['applicables'],
                                             $this->config['configMgmt']['siteURL'],
                                             $this->config['configMgmt']['listName'],
                                             '1',
                                             null,
                                             null
                                             );

        $this->configMgmt->create_global_mount($this->config['configMgmt']['globalMountPoint'].'2',
                                             $this->config['configMgmt']['applicables'],
                                             $this->config['configMgmt']['siteURL'],
                                             $this->config['configMgmt']['listName'],
                                             '2',
                                             'user',
                                             'pass'
                                             );

        $this->configMgmt->create_global_mount($this->config['configMgmt']['globalMountPoint'].'3',
                                             $this->config['configMgmt']['applicables'],
                                             $this->config['configMgmt']['siteURL'],
                                             $this->config['configMgmt']['listName'],
                                             '3',
                                             'user',
                                             'pass'
                                             );
        $this->configMgmt->set_user_credentials('$harepointAdmin', $this->config['sharepoint2010']['site']['user'], $this->config['sharepoint2010']['site']['password']);

        \OCP\Config::setAppValue('sharepoint', 'allow_personal_mounts', '1');

        //This method does not return nothing, but is usseful to check exceptions
        UserHooks::mount_storage(array('user'=>$this->config['global']['testUser']));
        //Add assert to validate test
        $this->assertTrue(true);

        $this->configMgmt->delete_user_mount($this->config['configMgmt']['userMountPoint'], $this->config['global']['testUser']);
        $this->configMgmt->delete_user_mount($this->config['configMgmt']['userMountPoint'].'2', $this->config['global']['testUser']);
        $mount = $this->find_global_mount_by_name($this->config['configMgmt']['globalMountPoint']);
        $this->configMgmt->delete_global_mount($mount['mount_id']);
        $mount = $this->find_global_mount_by_name($this->config['configMgmt']['globalMountPoint'].'2');
        $this->configMgmt->delete_global_mount($mount['mount_id']);
        $mount = $this->find_global_mount_by_name($this->config['configMgmt']['globalMountPoint'].'3');
        $this->configMgmt->delete_global_mount($mount['mount_id']);
    }

    public function find_global_mount_by_name($name){
        $gmounts = $this->configMgmt->get_global_mounts();
        foreach ($gmounts as $mount) {
            if($mount['mount_point'] === $name){
                return $mount;
            }
        }
    }
}
