<?php
/**
 * ownCloud
 *
 * @author Lukas Reschke <lukas@owncloud.com>
 * @copyright (C) 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\user_shibboleth\tests\controller;

use OCA\user_shibboleth\Controller\PasswordController;
use OCP\AppFramework\Http\DataResponse;
use Test\TestCase;
use OCP\Security\ISecureRandom;
use OCP\IL10N;
use OCP\IRequest;

/**
 * Class PasswordControllerTest
 *
 * @package OCA\user_shibboleth\tests\controller
 */
class PasswordControllerTest extends TestCase {
	/** @var IRequest */
	private $request;
	/** @var ISecureRandom */
	private $secureRandom;
	/** @var IL10N */
	private $l10n;
	/** @var PasswordController */
	private $passwordController;

	public function setUp() {
		parent::setUp();

		$this->request = $this->getMock('\OCP\IRequest');
		$this->secureRandom = $this->getMock('\OCP\Security\ISecureRandom');
		$this->l10n = $this->getMock('\OCP\IL10N');

		$this->passwordController = new PasswordController(
			'user_shibboleth',
			'testUser',
			$this->request,
			$this->secureRandom,
			$this->l10n
		);

		// FIXME: Hack to create the user for shib due to static code
		$createUserQuery= \OC_DB::prepare(
			'INSERT INTO `*PREFIX*shibboleth_user` (`eppn`, `email`, `displayname`, `webdav_password`) VALUES (?, ?, ?, NULL)'
		);
		$createUserQuery->execute(['testUser', 'test@owncloud.org', 'Just a test user']);
	}

	public function tearDown() {
		// FIXME: Hack to create the user for shib due to static code
		$createUserQuery= \OC_DB::prepare(
			'DELETE FROM`*PREFIX*shibboleth_user` WHERE `eppn` = ?'
		);
		$createUserQuery->execute(['testUser']);
		parent::tearDown();
	}

	public function testGetNewPasswordDefault() {
		$mediumStrengthGenerator = $this->getMock('\OCP\Security\ISecureRandom');
		$mediumStrengthGenerator
			->expects($this->at(0))
			->method('generate')
			->with(
				4,
				ISecureRandom::CHAR_DIGITS.ISecureRandom::CHAR_LOWER.ISecureRandom::CHAR_UPPER
			)
			->will($this->returnValue('1234'));
		$mediumStrengthGenerator
			->expects($this->at(1))
			->method('generate')
			->with(
				4,
				ISecureRandom::CHAR_DIGITS.ISecureRandom::CHAR_LOWER.ISecureRandom::CHAR_UPPER
			)
			->will($this->returnValue('5678'));
		$mediumStrengthGenerator
			->expects($this->at(2))
			->method('generate')
			->with(
				4,
				ISecureRandom::CHAR_DIGITS.ISecureRandom::CHAR_LOWER.ISecureRandom::CHAR_UPPER
			)
			->will($this->returnValue('9123'));

		$this->secureRandom
			->expects($this->exactly(3))
			->method('getMediumStrengthGenerator')
			->will($this->returnValue($mediumStrengthGenerator));

		$response = $this->passwordController->getNewPassword();
		$expected = new DataResponse(['1234', '5678', '9123']);
		$this->assertEquals($expected, $response);
	}


	public function testGetNewPasswordOdd() {
		$previous = \OC::$server->getConfig()->getSystemValue('non_shib_webdav_password_length');
		\OC::$server->getConfig()->setSystemValue('non_shib_webdav_password_length', 14);

		$mediumStrengthGenerator = $this->getMock('\OCP\Security\ISecureRandom');
		$mediumStrengthGenerator
			->expects($this->at(0))
			->method('generate')
			->with(
				4,
				ISecureRandom::CHAR_DIGITS.ISecureRandom::CHAR_LOWER.ISecureRandom::CHAR_UPPER
			)
			->will($this->returnValue('1234'));
		$mediumStrengthGenerator
			->expects($this->at(1))
			->method('generate')
			->with(
				4,
				ISecureRandom::CHAR_DIGITS.ISecureRandom::CHAR_LOWER.ISecureRandom::CHAR_UPPER
			)
			->will($this->returnValue('5678'));
		$mediumStrengthGenerator
			->expects($this->at(2))
			->method('generate')
			->with(
				4,
				ISecureRandom::CHAR_DIGITS.ISecureRandom::CHAR_LOWER.ISecureRandom::CHAR_UPPER
			)
			->will($this->returnValue('9123'));

		$this->secureRandom
			->expects($this->exactly(3))
			->method('getMediumStrengthGenerator')
			->will($this->returnValue($mediumStrengthGenerator));

		$response = $this->passwordController->getNewPassword();
		$expected = new DataResponse(['1234', '5678', '9123']);
		$this->assertEquals($expected, $response);

		\OC::$server->getConfig()->setSystemValue('non_shib_webdav_password_length', $previous);
	}

}
