<?php
/**
 * ownCloud
 *
 * @author Thomas Müller <deepdiver@owncloud.com>
 * @author Iago López Galeiras
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

use OCA\user_shibboleth\USER_SHIBBOLETH;

class ProvisioningTest extends \PHPUnit_Framework_TestCase {

	/**
	 * @var USER_SHIBBOLETH
	 */
	private $backend;

	/**
	 * @var string
	 */
	private $currentUser;

	public function setup() {
		\OC::$server->getappconfig()->setvalue("user_shibboleth", "shibboleth_active", true);

		$this->currentUser = uniqid('shib-user-') . '@example.com';
		$_SERVER["PHP_AUTH_USER"] = $this->currentUser;
		$_SERVER["eppn"] = $this->currentUser;
		$_SERVER["Shib-Session-ID"] = uniqid('shib');

		$this->backend = new USER_SHIBBOLETH();
	}

	public function testDeleteCredential() {
		$result = $this->backend->isSessionActive();
		$this->assertTrue($result);

		$this->backend->setWebDAVPassword($this->currentUser, '1234');
		$this->assertTrue($this->backend->checkIfUserHasWebDAVPassword($this->currentUser));

		$parameters = array('userid' => $this->currentUser);
		$return = OCA\user_shibboleth\Provisioning::deleteCredential($parameters);
		$this->assertTrue($return->succeeded());
		$this->assertFalse($this->backend->checkIfUserHasWebDAVPassword($this->currentUser));

		$parameters = array('userid' => "bogusUser");
		$return = OCA\user_shibboleth\Provisioning::deleteCredential($parameters);
		$this->assertEquals(\OC_API::RESPOND_NOT_FOUND, $return->getStatusCode());

		$parameters = array();
		$return = OCA\user_shibboleth\Provisioning::deleteCredential($parameters);
		$this->assertEquals(\OC_API::RESPOND_SERVER_ERROR, $return->getStatusCode());
	}

}
